﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;

public class HorrorEventTrigger : MonoBehaviour {

	private MeshRenderer meshRenderer;
	private Collider collider;

	public float delay = 0;
	public bool waitUntilUnseen = true;
	public bool once = true;

	public enum Trigger {
		Enter,
		Exit,
		Stay
	}

	public enum HorrorEffect {
		Move,
		Translate,
		AddForce,
		Hide,
		Show
	}

	[System.Serializable]
	public class HorrorEvent {
		public HorrorEffect horrorEffect;
		public GameObject target;
		public Vector3 vector;

		public void TriggerEvent() {
			switch (horrorEffect) {
				case HorrorEffect.Move:
					target.transform.position = vector;
					break;
				case HorrorEffect.Translate:
					target.transform.Translate(vector);
					break;
				case HorrorEffect.AddForce:
					target.GetComponent<Rigidbody>().AddForce(vector);
					break;
				case HorrorEffect.Hide:
					target.SetActive(false);
					break;
				case HorrorEffect.Show:
					target.SetActive(true);
					break;
				default:
					break;
			}
		}
	}

	[System.Serializable]
	public class HorrorEvents {
		public List<HorrorEvent> horrorEvents;
	}

	public HorrorEvents horrorEvents = new HorrorEvents();
	public Trigger trigger;

	void Awake() {
		meshRenderer = GetComponent<MeshRenderer>();
		collider = GetComponent<Collider>();
	}

	void OnTriggerEnter(Collider other) {
		if (other.tag == "Player") {
			print("TRIGGERD");
			StartCoroutine(CallEffects());
		}
	}

	IEnumerator CallEffects() {
		yield return new WaitForSeconds(delay);
		if (waitUntilUnseen) {
			while (true) {
				if (!meshRenderer.isVisible) {
					print("Turn around");
					foreach (HorrorEvent he in horrorEvents.horrorEvents) he.TriggerEvent();
					break;
				}
				yield return null;
			}
		} else foreach (HorrorEvent he in horrorEvents.horrorEvents) he.TriggerEvent();
		if (once) collider.enabled = false;
		yield return null;
	}
	
}


#if UNITY_EDITOR
[CustomPropertyDrawer(typeof(HorrorEventTrigger.HorrorEvents))]
public class HorrorEventsDrawer: PropertyDrawer {
	private const float k_LineHeight = 18;
	private const float k_Spacing = 5;

	public override void OnGUI(Rect position, SerializedProperty property, GUIContent label) {
		EditorGUI.BeginProperty(position, label, property);

		float x = position.x;
		float y = position.y;
		float width = position.width;

		EditorGUI.PrefixLabel(position, GUIUtility.GetControlID(FocusType.Passive), label);

		var indent = EditorGUI.indentLevel;
		EditorGUI.indentLevel = 0;

		var horrorEvents = property.FindPropertyRelative("horrorEvents");

		if (horrorEvents.arraySize > 0) {
			float effectWidth = .2f * width;
			float targetWidth = .2f * width;
			float vectorWidth = .5f * width;
			float buttonWidth = .1f * width;

			for (int i = 0; i < horrorEvents.arraySize; i++) {
				y += k_LineHeight + k_Spacing;
				var entry = horrorEvents.GetArrayElementAtIndex(i);

				float rowX = x;
				Rect effectRect = new Rect(rowX, y, effectWidth, k_LineHeight);
				rowX += effectWidth;

				Rect targetRect = new Rect(rowX, y, targetWidth, k_LineHeight);
				rowX += targetWidth;

				Rect vectorRect = new Rect(rowX, y, vectorWidth, k_LineHeight);
				rowX += vectorWidth;

				Rect buttonRect = new Rect(rowX, y, buttonWidth, k_LineHeight);

				EditorGUI.PropertyField(effectRect, entry.FindPropertyRelative("horrorEffect"), GUIContent.none);
				EditorGUI.PropertyField(targetRect, entry.FindPropertyRelative("target"), GUIContent.none);
				EditorGUI.PropertyField(vectorRect, entry.FindPropertyRelative("vector"), GUIContent.none);
				if (GUI.Button(buttonRect, "-")) {
					horrorEvents.DeleteArrayElementAtIndex(i);
					break;
				}
			}
		}
		y += k_LineHeight + k_Spacing;
		Rect addButtonRect = new Rect(x + width - 120, y, 60, k_LineHeight);
		if (GUI.Button(addButtonRect, "Add")) {
			horrorEvents.InsertArrayElementAtIndex(horrorEvents.arraySize);
		}

		EditorGUI.indentLevel = indent;
		EditorGUI.EndProperty();
	}

	public override float GetPropertyHeight(SerializedProperty property, GUIContent label) {
		SerializedProperty horrorEvents = property.FindPropertyRelative("horrorEvents");
		float lineAndSpace = k_LineHeight + k_Spacing;
		return 40 + (horrorEvents.arraySize * lineAndSpace) + lineAndSpace;
	}

}
#endif
